{% extends "admin/change_list.html" %}
{% load admin_list %}
{% load admin_urls %}
{% load i18n %}
{% load static %}

{% block title %}{% trans "List of forums" %}{% endblock %}

{% block extrahead %}
<link rel="stylesheet" type="text/css" href="{% static 'css/machina.admin_theme.min.css' %}" />
{{ block.super }}
{% endblock %}

{% block object-tools-items %}
<li><a href="{% url 'admin:forum_forum_editpermission_index' %}">{% trans "Global forum permissions" %}</a></li>
{{ block.super }}
{% endblock object-tools-items %}

{% block result_list %}
<form method="post" action="">{% csrf_token %}
  {% if action_form and actions_on_top and cl.full_result_count %}{% admin_actions %}{% endif %}
  <div id="forum-list">
    {% include "admin/forum/forum/change_list_table.html" %}
  </div>
  {% if action_form and actions_on_bottom and cl.full_result_count %}{% admin_actions %}{% endif %}
</form>
{% endblock %}
