//
// MACHINA BOARD: FORUM PROFILE PAGES
// --------------------------------------------------


.profile-sidebar{
  .card{
    override: hidden;

    .profile-avatar{
      .avatar.empty{
        display: block;
        height: 150px;
        background: $grayLighter;
        text-align: center;
        display: inline-block;

        i.far{
          margin-top: 50px;
        }
      }

      .avatar{
        width: 100%;
      }
    }
  }
}

.recent-posts{
  .card{
    padding-top: 0;
    padding-bottom: 0;

    .post{
      margin-bottom: 0;
      padding-top: 10px;
      border-bottom: 4px solid $grayLighter;

      .post-title{
        font-size: 16px;
      }
    }
  }
}
