{% extends 'board_base.html' %}
{% load i18n %}
{% load forum_conversation_tags %}

{% block sub_title %}{% trans "Disapprove post" %}{% endblock sub_title %}

{% block content %}
<div class="row">
  <div class="col-12">
  <div class="card topic-lock">
  <div class=card-header>
    <h3 class="m-0 card-title h5">{% trans "Disapprove post" %}</h3>
  </div>
  <div class="card-body">
    <div class="mb-3 warning-message">{% trans "Are you sure you want to disapprove this post? This post will be deleted." %}</div>
    <form action="" method="post">{% csrf_token %}
      <input type="hidden" name="post" value="yes" />
      <button role="submit" class="btn btn-warning">{% trans "Yes, I'm sure" %}</button>
      <a href="{% url 'forum_moderation:queued_post' post.pk %}" class="btn btn-secondary">{% trans "No, take me back" %}</a>
    </form>
  </div>
  </div>
  </div>
</div>
{% endblock content %}
