# -*- coding: utf-8 -*-
# Generated by Django 1.9.2 on 2016-02-25 04:15
from __future__ import unicode_literals

import django.core.validators
from django.db import migrations
import machina.models.fields


class Migration(migrations.Migration):

    dependencies = [
        ('forum_member', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='forumprofile',
            name='signature',
            field=machina.models.fields.MarkupTextField(blank=True, no_rendered_field=True, null=True, validators=[django.core.validators.MaxLengthValidator(255)], verbose_name='Signature'),
        ),
    ]
