//
// Variables
// --------------------------------------------------


//== Global values
//
//## Some global values used in all styles are defined here.

$baseUrl: '../';


// Default colors
//
// ## Some default colors...

$black:                 #000;
$blue:                  #049cdb;
$blueDark:             #0064cd;
$gray:                  #555;
$grayDark:             #333;
$grayDarker:           #222;
$grayLight:            #999;
$grayLighter:          #eee;
$green:                 #46a546;
$greenLight:           #7ac143;
$orange:                #f89406;
$pink:                  #c3325f;
$purple:                #7a43b6;
$red:                   #9d261d;
$redLight:             #B82010;
$white:                 #fff;
$yellow:                #ffc40d;
